/*
  This file is part of GerPhys.

  GerPhys is free software: you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerPhys is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerPhys.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "floatarea.h"


namespace GerPh {

  FloatArea::FloatArea() {
    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    fMap = NULL;
    fDrawing = NULL;
    fDrawArgs = NULL;
  } // FloatArea::FloatArea

  FloatArea::~FloatArea() {
    if (fMap) {
      fMap -> setDevice(NULL);
    } // if
  } // FloatArea::~FloatArea

  FloatMap* FloatArea::getMap() {
    return fMap;
  } // FloatArea::getMap

  void FloatArea::paintEvent(QPaintEvent *aEvent) {
    QWidget::paintEvent(aEvent);
    if (fMap && !fDrawing) {
      fDrawing = fMap -> getDrawing();
      fDrawArgs = fMap -> getDrawArgs();
    } // if
    if (fDrawing) {
      QPainter painter(this);
      (*fDrawing)(&painter, fDrawArgs);
      painter.end();
    } // if
  } // FloatArea::paintEvent

  void FloatArea::resizeEvent(QResizeEvent *aEvent) {
    if (fMap) {
      fMap -> rescale();
    } // if
    QWidget::resizeEvent(aEvent);
  } // FloatArea::resizeEvent

  void FloatArea::setMap(FloatMap *aValue) {
    if (aValue == fMap) {
      return;
    } // if
    if (fMap) {
      fMap -> setDevice(NULL);
    } // if
    fMap = aValue;
    fDrawing = NULL;
    fDrawArgs = NULL;
    if (fMap) {
      fMap -> setDevice(this);
    } // if
    repaint();
  } // FloatArea::setMap

} // GerPh
