/*
  This file is part of GerPhys.

  GerPhys is free software: you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerPhys is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerPhys.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef GERPH_VECTOR_H
#define GERPH_VECTOR_H


#include <gerq/gerqutilsbase.h>
#include <QList>
#include <QString>


namespace GerPh {


  class Vector {

    public:

      Vector(const Vector &aTemplate);
      Vector(int aDimension, double *aCoeff = NULL);
      virtual ~Vector();

      int getDimension() const;

      double getAt(int i) const;
      Vector *setAt(int i, double aValue);

      double getAbs();

      int checkDimension(const Vector &v, const QString &aCall) const;

      bool operator== (const Vector &v) { return isEqual(v); }
      Vector& operator= (const Vector &v) { return setEqual(v); }
      Vector& operator+= (const Vector &v) { return addVec(v); }
      Vector& operator-= (const Vector &v) { return subVec(v); }
      Vector& operator*= (double x) { return multDbl(x); }
      Vector& operator/= (double x) { return multDbl(1.0 / x); }

      typedef QList<Vector*> List;

    private:

      int fDimension;
      double *fCoeff;

      bool isEqual(const Vector &v) const;
      Vector& setEqual(const Vector &v);
      Vector& addVec(const Vector &v);
      Vector& subVec(const Vector &v);
      Vector& multDbl(double x);

  }; // Vector


  Vector operator+ (const Vector &v1, const Vector &v2);
  Vector operator- (const Vector &v1, const Vector &v2);
  double operator* (const Vector &v1, const Vector &v2);
  Vector operator* (double x, const Vector &v);
  Vector operator* (const Vector &v, double x);
  Vector operator/ (const Vector &v, double x);


} // GerPh


#endif
